#include "Colection.hpp"
#include <QSqlDatabase>
#include <QSqlQuery>
#include <QSqlError>
#include <QDesktopServices>
#include <QFile>
#include <QDir>
#include <QVariant>
#include <QTimer>
#include <QThread>

void Colection::add(const TrackInfoList trackList)
{
    for(int i = 0; i < trackList.count(); ++i)
    {
        int artisId = getArtistId(trackList.at(i).artistName);
        int albumId = getAlbumId(trackList.at(i).album);

        QSqlQuery qry(QSqlDatabase::database("INDEX"));
        qry.prepare("SELECT Id FROM Track WHERE Name = :name AND AlbumId = :aid AND ArtistId = :rid");
        qry.bindValue(":name",trackList.at(i).title);
        qry.bindValue(":aid",albumId);
        qry.bindValue(":rid",artisId);
        if(!qry.exec())
        {
            qFatal("Failed to retrive track ID");
        }
        int id = 0;
        if(!qry.next()) //This is a new Track
        {
            int temp = 0;
            QSqlQuery qry(QSqlDatabase::database("INDEX"));
            qry.prepare("SELECT Id FROM Track Order by Id");
            if(!qry.exec())
            {
                qFatal("Failed to count records");
            }
            while(qry.next())
            {
                int index = qry.value(0).toInt();
                if(temp < index)
                    break;
                else
                    ++temp;
            }
            id = temp;
        }
        else
        {
            id = qry.value(0).toInt();
            while(qry.next())
                id = qry.value(0).toInt();
            QSqlQuery deleteQuery(QSqlDatabase::database("INDEX"));
            deleteQuery.prepare("DELETE FROM Track WHERE Id=:id");
            deleteQuery.bindValue(":id",id);
            if(!deleteQuery.exec())
            {
                qFatal("Failed to drop track");
            }
        }
        QSqlQuery qryInsert(QSqlDatabase::database("INDEX"));
        qryInsert.prepare("INSERT INTO Track (Id,AlbumId,ArtistId,Path,Name,RunTime) VALUES(:id,:aid,:rid,:path,:na,:time)");
        qryInsert.bindValue(":id",id);
        qryInsert.bindValue(":aid",albumId);
        qryInsert.bindValue(":rid",artisId);
        qryInsert.bindValue(":path",trackList.at(i).path);
        qryInsert.bindValue(":na",trackList.at(i).title);
        qryInsert.bindValue(":time",trackList.at(i).runTime);
        if(!qryInsert.exec())
        {
            qFatal("Faild to add Track");
        }
    }
    emit ready();
}

void Colection::getArtists()
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT Id, Name FROM Artist Order by Name");
    if(!qry.exec())
    {
        qFatal("Failed to retrive artists");
    }
    QList<QPair<int,QString> > artists;
    while(qry.next())
    {
        QPair<int,QString> pair(qry.value(0).toInt(),qry.value(1).toString());
        artists.append(pair);
    }
    emit gotArtists(artists);
}

void Colection::getAlbums()
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT Id, Name FROM Album Order by Name");
    if(!qry.exec())
    {
        qFatal("Failed to retrive albums");
    }
    QList<QPair<int,QString> > albums;
    while(qry.next())
    {
        QPair<int,QString> pair(qry.value(0).toInt(),qry.value(1).toString());
        albums.append(pair);
    }
    emit gotAlbums(albums);
}

void Colection::getTracksFromAlbum(int albumId)
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT Id, Name FROM Track WHERE AlbumId = :aid Order by Name");
    qry.bindValue(":aid",albumId);
    if(!qry.exec())
    {
        qFatal("Failed to retrive tracks for album");
    }
    QList<QPair<int,QString> > tracks;
    while(qry.next())
    {
        QPair<int,QString> pair(qry.value(0).toInt(),qry.value(1).toString());
        tracks.append(pair);
    }
    emit gotTracks(tracks);
}

void Colection::getTracksFromArtist(int artistId)
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT Id, Name FROM Track WHERE ArtistId = :aid Order by Name");
    qry.bindValue(":aid",artistId);
    if(!qry.exec())
    {
        qFatal("Failed to retrive tracks for artist");
    }
    QList<QPair<int,QString> > tracks;
    while(qry.next())
    {
        QPair<int,QString> pair(qry.value(0).toInt(),qry.value(1).toString());
        tracks.append(pair);
    }
    emit gotTracks(tracks);
}

void Colection::getTrackInfo(int trackId)
{
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT AlbumId, ArtistId, Path, Name, RunTime FROM Track WHERE Id = :id");
    qry.bindValue(":id",trackId);
    if(!qry.exec())
    {
        qFatal("Failed to retrive track path");
    }
    int albumId = 0;
    int artistId = 0;
    TrackInfo info;
    while(qry.next())
    {
        albumId = qry.value(0).toInt();
        artistId = qry.value(1).toInt();
        info.path = qry.value(2).toString();
        info.title = qry.value(3).toString();
        info.runTime = qry.value(4).toString();
    }

    QSqlQuery albumQuery(QSqlDatabase::database("INDEX"));
    albumQuery.prepare("SELECT Name FROM Album WHERE Id = :id");
    albumQuery.bindValue(":id",albumId);
    if(!albumQuery.exec())
    {
        qFatal("Failed to retrive album name");
    }
    while(albumQuery.next())
        info.album = albumQuery.value(0).toString();

    QSqlQuery artistQuery(QSqlDatabase::database("INDEX"));
    artistQuery.prepare("SELECT Name FROM Artist WHERE Id = :id");
    artistQuery.bindValue(":id",artistId);
    if(!artistQuery.exec())
    {
        qFatal("Failed to retrive artist name");
    }
    while(artistQuery.next())
        info.artistName = artistQuery.value(0).toString();

    emit gotTrackInfo(info);
}

void Colection::getTracksInfoFromAlbum(int albumId)
{
    QSqlQuery albumQuery(QSqlDatabase::database("INDEX"));
    albumQuery.prepare("SELECT Name FROM Album WHERE Id = :id ");
    albumQuery.bindValue(":id",albumId);
    if(!albumQuery.exec())
    {
        qFatal("Failed to retrive album name");
    }
    QString albumName;
    while(albumQuery.next())
        albumName = albumQuery.value(0).toString();

    TrackInfoList list;
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT ArtistId, Path, Name, RunTime FROM Track WHERE AlbumId = :id Order by Name");
    qry.bindValue(":id",albumId);
    if(!qry.exec())
    {
        qFatal("Failed to retrive track path");
    }
    while(qry.next())
    {
        int artistId = 0;
        TrackInfo info;
        artistId = qry.value(0).toInt();
        info.path = qry.value(1).toString();
        info.title = qry.value(2).toString();
        info.runTime = qry.value(3).toString();
        info.album = albumName;

        QSqlQuery artistQuery(QSqlDatabase::database("INDEX"));
        artistQuery.prepare("SELECT Name FROM Artist WHERE Id = :id");
        artistQuery.bindValue(":id",artistId);
        if(!artistQuery.exec())
        {
            qFatal("Failed to retrive artist name");
        }
        while(artistQuery.next())
            info.artistName = artistQuery.value(0).toString();

        list.append(info);
    }
    emit gotTrackFromArtistAlbum(list);
}

void Colection::getTracksInfoFromArtist(int artistId)
{

    QSqlQuery artistQuery(QSqlDatabase::database("INDEX"));
    artistQuery.prepare("SELECT Name FROM Artist WHERE Id = :id");
    artistQuery.bindValue(":id",artistId);
    if(!artistQuery.exec())
    {
        qFatal("Failed to retrive artist name");
    }
    QString artistName;
    while(artistQuery.next())
        artistName = artistQuery.value(0).toString();

    TrackInfoList list;
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT AlbumId, Path, Name, RunTime FROM Track WHERE ArtistId = :id Order by Name");
    qry.bindValue(":id",artistId);
    if(!qry.exec())
    {
        qFatal("Failed to retrive track path");
    }
    while(qry.next())
    {
        int albumId = 0;
        TrackInfo info;
        albumId = qry.value(0).toInt();
        info.path = qry.value(1).toString();
        info.title = qry.value(2).toString();
        info.runTime = qry.value(3).toString();
        info.artistName = artistName;

        QSqlQuery albumQuery(QSqlDatabase::database("INDEX"));
        albumQuery.prepare("SELECT Name FROM Album WHERE Id = :id");
        albumQuery.bindValue(":id",albumId);
        if(!albumQuery.exec())
        {
            qFatal("Failed to retrive album name");
        }
        while(albumQuery.next())
            info.album = albumQuery.value(0).toString();

        list.append(info);
    }
    emit gotTrackFromArtistAlbum(list);
}

int Colection::getArtistId(QString artist)
{
    int temp = 0;
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT Id FROM Artist WHERE Name = :name");
    qry.bindValue(":name",artist);
    if(!qry.exec())
    {
        qFatal("Failed retrive artist ID");
    }
    if(qry.next())
        temp = qry.value(0).toInt();
    else
    {
        QSqlQuery qry(QSqlDatabase::database("INDEX"));
        qry.prepare("SELECT Id FROM Artist Order by Id");
        if(!qry.exec())
        {
            qFatal("Failed to count records");
        }
        while(qry.next())
        {
            int index = qry.value(0).toInt();
            if(temp < index)
                break;
            else
                ++temp;
        }
        QSqlQuery qryInsert(QSqlDatabase::database("INDEX"));
        qryInsert.prepare("INSERT INTO Artist (Id,Name) VALUES(:id,:na)");
        qryInsert.bindValue(":id",temp);
        qryInsert.bindValue(":na",artist);
        if(!qryInsert.exec())
        {
            qFatal("Faild to add Artist");
        }
    }
    return temp;
}

int Colection::getAlbumId(QString album)
{
    int temp = 0;
    QSqlQuery qry(QSqlDatabase::database("INDEX"));
    qry.prepare("SELECT Id FROM Album WHERE Name = :name");
    qry.bindValue(":name",album);
    if(!qry.exec())
    {
        qFatal("Failed retrive album ID");
    }
    if(qry.next())
        temp = qry.value(0).toInt();
    else
    {
        QSqlQuery qry(QSqlDatabase::database("INDEX"));
        qry.prepare("SELECT Id FROM Album Order by Id");
        if(!qry.exec())
        {
            qFatal("Failed to count records");
        }
        while(qry.next())
        {
            int index = qry.value(0).toInt();
            if(temp < index)
                break;
            else
                ++temp;
        }
        QSqlQuery qryInsert(QSqlDatabase::database("INDEX"));
        qryInsert.prepare("INSERT INTO Album (Id,Name) VALUES(:id,:na)");
        qryInsert.bindValue(":id",temp);
        qryInsert.bindValue(":na",album);
        if(!qryInsert.exec())
        {
            qFatal("Faild to add Album");
        }
    }
    return temp;
}

void Colection::connectToDataBase()
{
    QString dataBaseFile =
            QDesktopServices::storageLocation(QDesktopServices::MusicLocation);
    dataBaseFile.append("/VulcanoIndex.vInd");
    bool needToCreate = !QFile::exists(dataBaseFile);
    if(needToCreate)
    {
        QDir dir;
        dir.mkpath(QDesktopServices::storageLocation(
                QDesktopServices::MusicLocation));
    }
    QSqlDatabase db =  QSqlDatabase::addDatabase("QSQLITE","INDEX");
    db.setDatabaseName(dataBaseFile);
    if(!db.open())
        qFatal("Failed to open database");
    if(needToCreate)
    {
        QSqlQuery indexQuery(db);

        //creating Artist TABLE
        indexQuery.prepare("CREATE TABLE IF NOT EXISTS Artist"
                           "(Id INTEGER PRIMARY KEY,"
                           "Name VARCHAR(30))");
        if(!indexQuery.exec())
            qFatal("Failed to create Artist table.");
        //creating Album TABLE
        indexQuery.prepare("CREATE TABLE IF NOT EXISTS Album"
                           "(Id INTEGER PRIMARY KEY,"
                           "Name VARCHAR(30))");
        if(!indexQuery.exec())
            qFatal("Failed to create Album table.");
        //creating Track TABLE
        indexQuery.prepare("CREATE TABLE IF NOT EXISTS Track"
                           "(Id INTEGER PRIMARY KEY,"
                           "AlbumId INTEGER,"
                           "ArtistId INTEGER,"
                           "Path VARCHAR(30),"
                           "Name VARCHAR(30),"
                           "RunTime VARCHAR(10))");
        if(!indexQuery.exec())
            qFatal("Failed to create Track table.");
    }
    emit ready();
}

Colection::Colection()
    :QObject(0)
{
    timer = new QTimer();
    timer->setSingleShot(true);
    timer->setInterval(100);
    connect(timer,SIGNAL(timeout()),this,SLOT(connectToDataBase()));
    timer->start();
    myThread = new QThread;
    this->moveToThread(myThread);
    myThread->start();
}

Colection::~Colection()
{
    delete timer;
    QSqlDatabase db = QSqlDatabase::database("INDEX");
    db.close();
    myThread->exit();
    myThread->wait();
    delete myThread;
}
