#include "ColectionScanner.hpp"
#include "DirBuffer.hpp"
#include "DirScanner.hpp"
#include "FileBuffer.hpp"
#include "FileScanner.hpp"

TrackInfoList ColectionScanner::getFoundTrackInfo() const
{
    return fileBuffer->getTrackInfoList();
}

FileScanner* ColectionScanner::getFileScaner() const
{
    return fileScanner;
}

void ColectionScanner::starScan(QString dirPath,QStringList suffixList)
{
    currentSuffixList = suffixList;
    emit searchPath(dirPath);
    emit starFileSearch(currentSuffixList);
}

void ColectionScanner::cancelScan()
{
    dirScanner->stop();
    fileScanner->stop();
}

void ColectionScanner::dirScanCompleat()
{
    fileScanner->dirScanFinished();
}

void ColectionScanner::fileScaneCompleat()
{
    emit scanCompleat();
}

ColectionScanner::ColectionScanner(QObject *parent)
    :QObject(parent)
{
    dirBuffer = new DirBuffer;
    dirScanner = new DirScanner(dirBuffer);
    connect(this,SIGNAL(searchPath(QString)),dirScanner,SLOT(starDirScan(QString)),Qt::QueuedConnection);
    connect(dirScanner,SIGNAL(dirScanFinished()),this,SLOT(dirScanCompleat()),Qt::QueuedConnection);

    fileBuffer = new FileBuffer;

    fileScanner = new FileScanner(dirBuffer,fileBuffer);
    connect(this,SIGNAL(starFileSearch(QStringList)),fileScanner,SLOT(startScan(QStringList)));
    connect(fileScanner,SIGNAL(scannFinished()),this,SLOT(fileScaneCompleat()));
}

ColectionScanner::~ColectionScanner()
{
    delete fileScanner;
    delete fileBuffer;
    dirScanner->stop();
    delete dirScanner;
    delete dirBuffer;
}
