#include "FileScanner.hpp"
#include "DirBuffer.hpp"
#include "FileBuffer.hpp"
#include <QThread>
#include <QTimer>
#include <QFileInfo>
#include <phonon/MediaObject>
#include <QEventLoop>
#include <QTime>

const int TimeOut = 8000;

bool waitForLoad(Phonon::MediaObject* object,int timeOut) // Wait for the media to load or timeout
{
    QEventLoop loop;
    QTimer timer;
    timer.setSingleShot(true);
    timer.setInterval(timeOut);
    QObject::connect(&timer,SIGNAL(timeout()),&loop,SLOT(quit()));
    QObject::connect(object,SIGNAL(stateChanged(Phonon::State,Phonon::State)),&loop,SLOT(quit()));
    timer.start();
    loop.exec();
    return object->state() == Phonon::StoppedState;
}

void FileScanner::stop()
{
    run = false;
}

void FileScanner::dirScanFinished()
{
    dirScannRunning = false;
}

void FileScanner::startScan(QStringList suffixList)
{
    run = true;
    dirScannRunning = true;
    currentSuffixList = suffixList;
    timer->start(100);
}

void FileScanner::scanNext()
{
    if(!run) return;
    bool ok = true;
    QString filePath = myDirBuffer->takeFirst(ok);
    if(ok)
    {
        QString scanning = QString("Scanning: %1").arg(filePath);
        emit scanningFile(scanning);
        QFileInfo fileInfo(filePath);
        if(currentSuffixList.contains(fileInfo.suffix()))
        {
            Phonon::MediaObject media;
            Phonon::MediaSource source(filePath);
            media.setCurrentSource(source);
            if(waitForLoad(&media,TimeOut))
            {
                QString artist = media.metaData(Phonon::ArtistMetaData).join("/").trimmed();
                if(artist.isEmpty())
                    artist = "Uknown";
                QString album = media.metaData(Phonon::AlbumMetaData).join("/").trimmed();
                if(album.isEmpty())
                    album = "Unknown";
                QString title = media.metaData(Phonon::TitleMetaData).join("/").trimmed();
                if(title.isEmpty())
                    title = fileInfo.fileName().split(".").at(0);
                int seckonds = media.totalTime() / 1000;
                int minutes = seckonds/60;
                seckonds = seckonds%60;
                QString runTime;
                if(seckonds < 10)
                    runTime = QString("%1:0%2").arg(minutes).arg(seckonds);
                else
                    runTime = QString("%1:%2").arg(minutes).arg(seckonds);
                TrackInfo info;
                info.artistName = artist;
                info.album = album;
                info.title = title;
                info.path = filePath;
                info.runTime = runTime;
                myFileBuffer->append(info);

            }
            else
                qDebug("Load Error");
        }
        timer->start(10);
    }
    else
    {
        if(dirScannRunning)
            timer->start(100);
        else
            emit scannFinished();
    }
}


FileScanner::FileScanner(DirBuffer* dirBuffer,FileBuffer* buffer)
    :QObject(0),myDirBuffer(dirBuffer),myFileBuffer(buffer)
{
    myThread = new QThread;
    timer = new QTimer(this);
    timer->setSingleShot(true);
    connect(timer,SIGNAL(timeout()),this,SLOT(scanNext()));
    run = true;
    dirScannRunning = true;
    this->moveToThread(myThread);
    myThread->start();
}

FileScanner::~FileScanner()
{
    //delete timer;
    myThread->exit();
    myThread->wait();
    delete myThread;
}
