#include "ScanDialog.hpp"
#include "ColectionScanner.hpp"
#include "FileScanner.hpp"
#include <QFileDialog>
#include <phonon/BackendCapabilities>

TrackInfoList ScanDialog::getResault() const
{
    return scanner->getFoundTrackInfo();
}

void ScanDialog::startScan()
{
    QString path = QFileDialog::getExistingDirectory(this,"Select directory");

    if(!path.isEmpty())
    {
        QStringList suffixList;
        //        foreach(QString mime,Phonon::BackendCapabilities::availableMimeTypes())
        //        {
        //            if(mime.contains("audio/"))
        //            {
        //                suffixList << mime;
        //            }
        //        }

        suffixList << "mp3" << "ogg";

        ui.progressBar->setMinimum(0);
        ui.progressBar->setMaximum(0);
        ui.scanOutput->setText("");

        scanner->starScan(path,suffixList);
        ui.scanButton->setEnabled(false);
        ui.cancelButton->setEnabled(true);
        ui.insertButton->setEnabled(false);
    }
}

void ScanDialog::cancelScan()
{
    scanner->cancelScan();
    ui.cancelButton->setEnabled(false);
    ui.scanButton->setEnabled(true);
    ui.insertButton->setEnabled(false);
    ui.scanOutput->setText(ui.scanOutput->toPlainText().append("Canceled"));
    ui.progressBar->setMinimum(0);
    ui.progressBar->setMaximum(1);
    ui.progressBar->setValue(0);
    ui.scanOutput->moveCursor(QTextCursor::End);
    ui.scanOutput->ensureCursorVisible();
}

void ScanDialog::scanFinished()
{
    ui.cancelButton->setEnabled(false);
    ui.scanButton->setEnabled(true);
    ui.insertButton->setEnabled(true);
    ui.scanOutput->setText(ui.scanOutput->toPlainText().append("Finished"));
    ui.progressBar->setMinimum(0);
    ui.progressBar->setMaximum(1);
    ui.progressBar->setValue(1);
    ui.scanOutput->moveCursor(QTextCursor::End);
    ui.scanOutput->ensureCursorVisible();
}

void ScanDialog::insertResaults()
{
    accept();
}

void ScanDialog::message(QString message)
{
    if(ui.scanOutput->toPlainText().count() > 1800)
    {
        QString text = ui.scanOutput->toPlainText();
        text.remove(0,1000);
        ui.scanOutput->setText(text.append(message).append("\n"));
    }
    else
        ui.scanOutput->setText(ui.scanOutput->toPlainText().append(message).append("\n"));
    ui.scanOutput->moveCursor(QTextCursor::End);
    ui.scanOutput->ensureCursorVisible();
}

ScanDialog::ScanDialog(QWidget *parent)
    :QDialog(parent)
{
    ui.setupUi(this);
    scanner = new ColectionScanner(this);
    connect(scanner->getFileScaner(),SIGNAL(scanningFile(QString)),this,SLOT(message(QString)),Qt::QueuedConnection);
    connect(scanner,SIGNAL(scanCompleat()),this,SLOT(scanFinished()));
    connect(ui.scanButton,SIGNAL(clicked()),this,SLOT(startScan()));
    connect(ui.cancelButton,SIGNAL(clicked()),this,SLOT(cancelScan()));
    connect(ui.insertButton,SIGNAL(clicked()),this,SLOT(insertResaults()));
}

void ScanDialog::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui.retranslateUi(this);
        break;
    default:
        break;
    }
}
