#include "ArtistAlbumItem.hpp"
#include <QGraphicsSceneMouseEvent>
#include <QDrag>
#include <QMimeData>
#include <QApplication>
#include <QPainter>

ColectionItem* ArtistAlbumItem::newInstance(qreal width,int index,QString text,
                                            QGraphicsItem* parent)
{
    return new ArtistAlbumItem(width,index,text,isAlbum,parent);
}

void ArtistAlbumItem::setIsAlbum(bool album)
{
    isAlbum = album;
}

void ArtistAlbumItem::paint(QPainter* painter,const QStyleOptionGraphicsItem* option,QWidget* widget)
{
    ColectionItem::paint(painter,option,widget);
    if(isAlbum)
        painter->drawPixmap(2,2,QPixmap(":/eyecandy/album.png"));
    else
        painter->drawPixmap(2,2,QPixmap(":/eyecandy/artist.png"));
}

void ArtistAlbumItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
    event->accept();
    dragStart = event->pos();
}

void ArtistAlbumItem::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
{
    event->accept();
    emit clicked(myIndex);
}

void ArtistAlbumItem::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
{
    event->accept();
    if((event->pos() - dragStart).manhattanLength() < QApplication::startDragDistance() &&
       event->button() != Qt::LeftButton)
        return;
    QDrag* drag = new QDrag(event->widget());
    QMimeData *mime = new QMimeData;
    drag->setMimeData(mime);
    if(isAlbum)
        mime->setData("VulcanoPlayer/AlbumIndex",QByteArray::number(myIndex));
    else
        mime->setData("VulcanoPlayer/ArtistIndex",QByteArray::number(myIndex));
    if(isAlbum)
        drag->setPixmap(QPixmap(":/eyecandy/album.png"));
    else
        drag->setPixmap(QPixmap(":/eyecandy/artist.png"));
    drag->setHotSpot(QPoint(10,10));
    drag->exec();
}

ArtistAlbumItem::ArtistAlbumItem(qreal width,int index,QString text,bool album,QGraphicsItem* parent)
    :ColectionItem(width,index,text,parent),isAlbum(album)
{
}
