#include "ColectionItem.hpp"
#include <QPainter>
#include <QGraphicsSceneMouseEvent>
#include <QGraphicsSceneHoverEvent>
#include <QPropertyAnimation>
#include <QFontMetrics>
#include <QCursor>
#include <QLinearGradient>


QRectF ColectionItem::boundingRect() const
{
    return QRectF(0.0f,0.0f,myWidth,25.0f);
}

qreal ColectionItem::getWidth() const
{
    return myWidth;
}

qreal ColectionItem::getVPos() const
{
    return pos().y();
}

qreal ColectionItem::getHPos() const
{
    return pos().x();
}

void ColectionItem::setWidth(qreal width)
{
    myWidth = width;
    update();
}

void ColectionItem::setVPos(qreal vpos)
{
    setPos(pos().x(),vpos);
}

void ColectionItem::setHPos(qreal hpos)
{
    setPos(hpos,pos().y());
}

void ColectionItem::slideUp()
{
    setPos(pos().x(),pos().y() - boundingRect().height() - 4.0f);
}

void ColectionItem::slideDown()
{
    setPos(pos().x(),pos().y() + boundingRect().height() + 4.0f);
}

QString ColectionItem::getText() const
{
    return myText;
}

void ColectionItem::paint(QPainter* painter,const QStyleOptionGraphicsItem*,QWidget*)
{
    painter->setRenderHint(QPainter::Antialiasing);
    painter->setPen(QColor(55,55,55));
    QLinearGradient gradient(boundingRect().topRight(),boundingRect().bottomRight());
    if(hover)
    {
        gradient.setColorAt(0.0f,QColor(255,255,255,255));
        gradient.setColorAt(0.3f,QColor(255,255,255,0));
        gradient.setColorAt(0.7f,QColor(255,255,255,0));
        gradient.setColorAt(1.0f,QColor(255,255,255,255));
    }
    else
    {
        gradient.setColorAt(0.0f,QColor(255,255,255,15));
        gradient.setColorAt(0.2f,QColor(255,255,255,55));
        gradient.setColorAt(1.0f,QColor(255,255,255,155));
    }
    painter->drawRoundedRect(boundingRect(),12.0f,12.0f);
    painter->setBrush(gradient);
    painter->drawRoundedRect(boundingRect(),12.0f,12.0f);

    if(hover)
        painter->setPen(Qt::darkGray);
    else
        painter->setPen(QColor(220,220,220));

    QFontMetrics metrics(painter->font());
    QString drawnText = metrics.elidedText(myText,Qt::ElideMiddle,
                                           static_cast<int>(boundingRect().width()) - 35);
    painter->drawText(30,0,static_cast<int>(boundingRect().width()) - 35,
                      boundingRect().height(),Qt::AlignLeft|Qt::AlignVCenter,drawnText);
}

void ColectionItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
    event->accept();
    if(event->button() == Qt::LeftButton)
        emit clicked(myIndex);
}

void ColectionItem::hoverEnterEvent(QGraphicsSceneHoverEvent* event)
{
    event->accept();
    hover = true;
    update();
}

void ColectionItem::hoverLeaveEvent(QGraphicsSceneHoverEvent* event)
{
    event->accept();
    hover = false;
    update();
}

ColectionItem* ColectionItem::newInstance(qreal width,int index,QString text,QGraphicsItem* parent)
{
    return new ColectionItem(width,index,text,parent);
}

ColectionItem::ColectionItem(qreal width,int index,QString text,QGraphicsItem* parent)
    :QGraphicsObject(parent),myWidth(width),myIndex(index),myText(text),hover(false)
{
    setAcceptHoverEvents(true);
    setCursor(QCursor(Qt::PointingHandCursor));
}
