#ifndef COLECTIONITEMDISPLAY_HPP
#define COLECTIONITEMDISPLAY_HPP

#include <QGraphicsObject>

class ColectionItem;
class ScrollButton;
class SlideIndycator;
class QSequentialAnimationGroup;

class ColectionItemDisplay : public QGraphicsObject
{
    Q_OBJECT
public:
    explicit ColectionItemDisplay(ColectionItem* prototype,qreal width,qreal height,
                                  QGraphicsItem* parent = 0);
    ~ColectionItemDisplay();

    QRectF boundingRect() const;

    void populate(QList<QPair<int,QString> > list);
    void clear();
    void setSize(qreal width,qreal height);

protected:
    void paint(QPainter* painter,const QStyleOptionGraphicsItem* option,QWidget* widget);
    void wheelEvent(QGraphicsSceneWheelEvent* event);

signals:
    void itemClicked(int index);

public slots:
    void scrollUp();
    void scrollDown();
    void scroll(int ticks);
    void setFilter(const QString& filter);
    void aniStoped();

private:
    void setSlideIndycator();
    void filterItems();

    ColectionItem* myPrototype;
    QList<ColectionItem*> items;
    QList<ColectionItem*> filteredItems;

    qreal myWidth;
    qreal myHeight;

    ScrollButton* up;
    ScrollButton* down;
    SlideIndycator* indycator;
    QString currentFilter;
    QSequentialAnimationGroup* group;
};

#endif // COLECTIONITEMDISPLAY_HPP
