#include "ColectionToolBar.hpp"
#include <QPainter>
#include <QLinearGradient>
#include "GraphicsToolButton.hpp"
#include "VolumeControl.hpp"

QRectF ColectionToolBar::boundingRect() const
{
    return QRectF(0.0f,0.0f,myWidth,48.0f);
}

GraphicsToolButton* ColectionToolBar::addToolButtonLeft(QPixmap icon)
{
    GraphicsToolButton* button = new GraphicsToolButton(icon,this);
    leftButtons.append(button);
    arrangeButtons();
    return button;
}

GraphicsToolButton* ColectionToolBar::addToolButtonRight(QPixmap icon)
{
    GraphicsToolButton* button = new GraphicsToolButton(icon,this);
    rightButtons.append(button);
    arrangeButtons();
    return button;
}


VolumeControl* ColectionToolBar::getVolumeControl() const
{
    return myControl;
}

void ColectionToolBar::setWidth(qreal width)
{
    myWidth = width;
    arrangeButtons();
}

void ColectionToolBar::paint(QPainter* painter,const QStyleOptionGraphicsItem*,QWidget*)
{
    QLinearGradient gradient(boundingRect().topLeft(),boundingRect().bottomLeft());
    gradient.setColorAt(0.0f,QColor(125,125,125,255));
    gradient.setColorAt(0.15f,QColor(255,255,255,255));
    gradient.setColorAt(0.85f,QColor(255,255,255,255));
    gradient.setColorAt(1.0f,QColor(125,125,125,255));
    painter->setBrush(QBrush(gradient));
    painter->setPen(Qt::transparent);
    painter->drawRect(boundingRect());
}

void ColectionToolBar::arrangeButtons()
{
    for(int i = 0; i < leftButtons.count(); ++i)
        leftButtons.at(i)->setPos(48.0f * i,0.0f);
    for(int i = 0; i < rightButtons.count(); ++i)
        rightButtons.at(i)->setPos(myWidth - 48.0f - (i*48.0f),0.0f);
    myControl->setPos(boundingRect().center().x() - 90.0f,0.0f);
}

ColectionToolBar::ColectionToolBar(qreal width,QGraphicsItem* parent)
    :QGraphicsItem(parent),myWidth(width)
{
    myControl = new VolumeControl(this);
    myControl->setPos(boundingRect().center().x()  - 90.0f,0.0f);
}
