#include "FilterInput.hpp"
#include <QPainter>
#include <QLineEdit>
#include <QGraphicsProxyWidget>
#include "ClearItem.hpp"

QRectF FilterInput::boundingRect() const
{
    return QRectF(0.0f,0.0f,myWidth,myHeigh);
}

void FilterInput::resize(qreal w,qreal h)
{
    myWidth = w;
    myHeigh = h;
    proxy->resize(w,h);
    item->setPos(boundingRect().right() - item->boundingRect().width() - 2.0f,
                 boundingRect().top() + ((boundingRect().height() - item->boundingRect().height()))/2);
    update();
}

void FilterInput::paint(QPainter* painter,const QStyleOptionGraphicsItem*,QWidget*)
{
    painter->setPen(Qt::transparent);
    painter->setBrush(QColor(15,15,15));
    painter->drawRect(boundingRect());
}

void FilterInput::filterChanged()
{
    emit applayFilter(edit->text());
}

void FilterInput::clearFilter()
{
    edit->setText("");
    emit applayFilter(edit->text());
}

FilterInput::FilterInput(qreal width,qreal heigh,QGraphicsItem* parent)
    :QGraphicsObject(parent),myWidth(width),myHeigh(heigh)
{
    proxy = new QGraphicsProxyWidget(this);
    edit = new QLineEdit();
    edit->setStyleSheet("QLineEdit{"
        "background-color: rgb(125,125,125);"
        "}");
    proxy->setWidget(edit);
    proxy->setZValue(0);
    proxy->setPos(0.0f,0.0f);
    connect(edit,SIGNAL(textChanged(QString)),this,SLOT(filterChanged()));

    item = new ClearItem(this);
    item->setPos(boundingRect().right() - item->boundingRect().width(),
                 boundingRect().top() + ((boundingRect().height() - item->boundingRect().height()))/2);
    connect(item,SIGNAL(clicked()),this,SLOT(clearFilter()));
}
