#include "GraphicsToolButton.hpp"
#include <QPainter>
#include <QGraphicsSceneMouseEvent>
#include <QGraphicsSceneHoverEvent>
#include <QCursor>

QRectF GraphicsToolButton::boundingRect() const
{
    return QRectF(0.0f,0.0f,48.0f,48.0f);
}

void GraphicsToolButton::setEnabled(bool enabled_)
{
    enabled = enabled_;
}

void GraphicsToolButton::paint(QPainter* painter,const QStyleOptionGraphicsItem*,QWidget*)
{
    if(enabled)
        painter->drawPixmap(0,0,image);
    else
        painter->drawPixmap(0,0,disabledImage);
}

void GraphicsToolButton::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
    event->accept();
    if(!enabled) return;
    if(event->button() == Qt::LeftButton)
    {
        emit clicked();
    }
}

void GraphicsToolButton::hoverEnterEvent(QGraphicsSceneHoverEvent* event)
{
    event->accept();
    if(!enabled) return;
    hover = true;
    update();
}

void GraphicsToolButton::hoverLeaveEvent(QGraphicsSceneHoverEvent* event)
{
    event->accept();
    if(!enabled) return;
    hover = false;
    update();
}


GraphicsToolButton::GraphicsToolButton(QPixmap icon,QGraphicsItem* parent)
    :QGraphicsObject(parent)
{
    setAcceptHoverEvents(true);

    image = icon;

    QImage disabled = icon.toImage();
    int pixels = disabled.width() * disabled.height();
    unsigned int *data = (unsigned int *)disabled.bits();
    for (int i = 0; i < pixels; ++i)
    {
        int val = qGray(data[i]);
        data[i] = qRgba(val, val, val, qAlpha(data[i]));
    }

    disabledImage = QPixmap::fromImage(disabled);

    hover = false;
    enabled = true;
    setCursor(QCursor(Qt::PointingHandCursor));
}
