#include "ScrollButton.hpp"
#include <QPainter>
#include <QGraphicsSceneMouseEvent>
#include <QTimer>

QRectF ScrollButton::boundingRect() const
{
    return QRectF(0.0f,0.0f,10.0f,10.0f);
}

void ScrollButton::paint(QPainter* painter,const QStyleOptionGraphicsItem*,QWidget*)
{
    QPainterPath path;
    if(myDirection == Scroll::up)
    {
        path.moveTo(boundingRect().center().x(),boundingRect().top());
        path.lineTo(boundingRect().bottomLeft());
        path.lineTo(boundingRect().bottomRight());
        path.lineTo(boundingRect().center().x(),boundingRect().top());
    }
    else
    {
        path.lineTo(boundingRect().topRight());
        path.lineTo(boundingRect().center().x(),boundingRect().bottom());
        path.lineTo(boundingRect().topLeft());
    }
    painter->setRenderHint(QPainter::Antialiasing);
    painter->setPen(QColor(55,55,55));
    QLinearGradient gradient(boundingRect().topRight(),boundingRect().bottomRight());
    gradient.setColorAt(0.0f,QColor(255,255,255,0));
    gradient.setColorAt(0.1f,QColor(255,255,255,0));
    gradient.setColorAt(1.0f,QColor(255,255,255,155));
    painter->setBrush(gradient);
    painter->drawPath(path);
}

void ScrollButton::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
    event->accept();
    if(event->button() == Qt::LeftButton)
    {
        emit scroll();
        timer->start();
    }
}

void ScrollButton::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
{
    event->accept();
    if(event->button() == Qt::LeftButton)
        timer->stop();
}

void ScrollButton::timerTick()
{
    emit scroll();
}

ScrollButton::ScrollButton(Scroll::dir direction,QGraphicsItem* parent) :
    QGraphicsObject(parent),myDirection(direction)
{
    timer = new QTimer(this);
    timer->setInterval(50);
    connect(timer,SIGNAL(timeout()),this,SLOT(timerTick()));
    setZValue(2.0f);
}
