#include "SlideIndycator.hpp"
#include <QPainter>
#include <QGraphicsSceneMouseEvent>
#include <QLinearGradient>

QRectF SlideIndycator::boundingRect() const
{
    return QRectF(0.0f,0.0f,10.0f,myHeight);
}

void SlideIndycator::setParametres(qreal heigh,int ticks,int tick)
{
    myHeight = heigh;
    tickCount = ticks;
    tickSize = (myHeight / static_cast<qreal>(ticks));

    currentTick = tick;
    update();
}

void SlideIndycator::tickUp()
{
    if(currentTick > 0)
        --currentTick;
    update();
}

void SlideIndycator::tickDown()
{
    if(currentTick < tickCount - 1)
        ++currentTick;
    update();
}

void SlideIndycator::paint(QPainter* painter,const QStyleOptionGraphicsItem*,QWidget*)
{
    painter->setPen(QColor(55,55,55));
    painter->setRenderHint(QPainter::Antialiasing);
    QLinearGradient gradient(boundingRect().topRight(),boundingRect().topLeft());
    gradient.setColorAt(0.0f,QColor(255,255,255,125));
    gradient.setColorAt(0.45f,QColor(255,255,255,75));
    gradient.setColorAt(0.75f,QColor(0,0,0,0));
    painter->setBrush(QBrush(QColor(15,15,15)));
    painter->drawRoundedRect(boundingRect(),5.0f,5.0f);
    painter->setBrush(gradient);
    painter->drawRoundedRect(boundingRect(),5.0f,5.0f);
    QLinearGradient gradient2(boundingRect().topLeft(),boundingRect().topRight());
    gradient2.setColorAt(0.0f,QColor(255,255,255,225));
    gradient2.setColorAt(0.5f,QColor(255,255,255,0));
    gradient2.setColorAt(1.0f,QColor(255,255,255,100));
    painter->setBrush(gradient2);
    painter->setPen(Qt::transparent);
    if(tickSize < 2.0f)
    {
        painter->drawRoundedRect(0,currentTick * tickSize,boundingRect().width(),
                          2.0f,5.0f,5.0f);
    }
    else
        painter->drawRoundedRect(0,currentTick * tickSize,boundingRect().width(),
                          tickSize,5.0f,5.0f);
}

void SlideIndycator::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
    event->accept();
    qreal parentPos = 0.0f;
    if(parentItem())
        parentPos = parentItem()->pos().y();
    qreal posAfter = event->scenePos().y() - pos().y() - parentPos;
    qreal currentPos = currentTick * tickSize;
    if(currentPos > posAfter)
    {
        int dTick =static_cast<int>((currentPos - posAfter)/tickSize);
        emit scroll(dTick);
    }
    else
    {
        int dTick =static_cast<int>((posAfter - currentPos)/tickSize) * (-1);
        emit scroll(dTick);
    }
}

void SlideIndycator::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
{
    event->accept();
    qreal parentPos = 0.0f;
    if(parentItem())
        parentPos = parentItem()->pos().y();
    qreal posAfter = event->scenePos().y() - pos().y() - parentPos;
    qreal currentPos = currentTick * tickSize;
    if(currentPos > posAfter)
    {
        int dTick =static_cast<int>((currentPos - posAfter)/tickSize);
        emit scroll(dTick);
    }
    else
    {
        int dTick =static_cast<int>((posAfter - currentPos)/tickSize) * (-1);
        emit scroll(dTick);
    }
}

SlideIndycator::SlideIndycator(qreal height,QGraphicsItem* parent)
    :QGraphicsObject(parent),myHeight(height),tickCount(1),currentTick(0),tickSize(height)
{
}
