#include "VolumeSlider.hpp"
#include <QPainter>
#include <QGraphicsSceneMouseEvent>
#include <QLinearGradient>
#include <QCursor>

QRectF VolumeSlider::boundingRect() const
{
    return QRectF(0.0f,0.0f,100.0f,48.0f);
}

void VolumeSlider::setValue(qreal newVolumne)
{
    value = newVolumne;
    update();
}

void VolumeSlider::paint(QPainter* painter,const QStyleOptionGraphicsItem*,QWidget*)
{
    painter->setRenderHint(QPainter::Antialiasing);
    painter->setPen(QColor(55,55,55));
    painter->drawPath(path);
    painter->setPen(Qt::transparent);
    painter->setClipPath(path);


    QLinearGradient gradient(boundingRect().topRight(),boundingRect().bottomRight());
    gradient.setColorAt(0.0f,QColor(0,0,0,15));
    gradient.setColorAt(0.2f,QColor(0,0,0,55));
    gradient.setColorAt(1.0f,QColor(0,0,0,155));
    painter->setBrush(gradient);
    qreal precent = value;
    painter->drawRect(0,0,boundingRect().width() * precent,boundingRect().height());
}

void VolumeSlider::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
    event->accept();
    qreal posX = this->pos().x();
    if(parentItem())
        posX += parentItem()->pos().x();
    value = (event->scenePos().x() - posX) / 100.0f;
    if(value > 1.0f)
        value = 1.0f;
    if(value < 0.0f)
        value = 0.0f;
    emit valueChanged(value);
    update();
}

void VolumeSlider::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
{
    event->accept();
    qreal posX = this->pos().x();
    if(parentItem())
        posX += parentItem()->pos().x();
    value = (event->scenePos().x() - posX) / 100.0f;
    if(value > 1.0f)
        value = 1.0f;
    if(value < 0.0f)
        value = 0.0f;
    emit valueChanged(value);
    update();
}

VolumeSlider::VolumeSlider(QGraphicsItem* parent)
    :QGraphicsObject(parent),value(1.0f)
{
    path.addRoundedRect(0.0f,6.0f,16.0f,36.0f,4.0f,4.0f);
    path.addRoundedRect(21.0f,6.0f,16.0f,36.0f,4.0f,4.0f);
    path.addRoundedRect(42.0f,6.0f,16.0f,36.0f,4.0f,4.0f);
    path.addRoundedRect(63.0f,6.0f,16.0f,36.0f,4.0f,4.0f);
    path.addRoundedRect(84.0f,6.0f,16.0f,36.0f,4.0f,4.0f);
        setCursor(QCursor(Qt::PointingHandCursor));
}
