#ifndef PLAYLISTDISPLAY_HPP
#define PLAYLISTDISPLAY_HPP

#include <QGraphicsObject>
#include "../Colection/TrackInfo.hpp"

class PlayListItem;
class ScrollButton;
class SlideIndycator;

class PlayListDisplay : public QGraphicsObject
{
    Q_OBJECT
public:
    explicit PlayListDisplay(qreal width,qreal height,QGraphicsItem* parent = 0);
    ~PlayListDisplay();

    QRectF boundingRect() const;

    void clear();
    void setSize(qreal width,qreal height);

    QString getCurrentPath();
    QString next();
    QString previous();

protected:
    void paint(QPainter* painter,const QStyleOptionGraphicsItem* option,QWidget* widget);
    void wheelEvent(QGraphicsSceneWheelEvent* event);
    void dragEnterEvent(QGraphicsSceneDragDropEvent* event);
    void dragLeaveEvent(QGraphicsSceneDragDropEvent* event);
    void dragMoveEvent(QGraphicsSceneDragDropEvent* event);
    void dropEvent(QGraphicsSceneDragDropEvent* event);

signals:
    void itemClicked(QString path);
    void requestTrack(int index,int aftere);
    void requestTracks(bool album,int index,int aftere);

public slots:
    void populate(TrackInfoList list, int index = -1);
    void scrollUp();
    void scrollDown();
    void scroll(int ticks);
    void itemClickedAction(PlayListItem* item);

private:
    void setSlideIndycator();

    void append(TrackInfoList list);
    void insertAfter(TrackInfoList list,int index);
    void setDropIndycator(QPointF pos);
    void markCurrent();
    void transferItem(PlayListItem* item,int after);

private slots:
    void removeItem(PlayListItem* item);

private:
    QList<PlayListItem*> items;

    qreal myWidth;
    qreal myHeight;

    ScrollButton* up;
    ScrollButton* down;
    SlideIndycator* indycator;
    QString currentFilter;
    qreal dropLineY;
    int dropIndex;
    bool drawDropLine;
    int currentListIndex;
};

#endif // PLAYLISTDISPLAY_HPP
