#include "PlayListItem.hpp"
#include <QPainter>
#include <QLinearGradient>
#include <QGraphicsSceneMouseEvent>
#include <QCursor>
#include <QDrag>
#include <QDataStream>
#include <QApplication>
#include <QMimeData>

QString PlayListItem::getPath() const
{
    return myTrackInfo.path;
}

TrackInfo PlayListItem::getTrackInfo() const
{
    return myTrackInfo;
}

void PlayListItem::setCurrent(bool current_)
{
    current = current_;
    update();
}

void PlayListItem::paint(QPainter* painter,const QStyleOptionGraphicsItem*,QWidget*)
{
    painter->setRenderHint(QPainter::Antialiasing);
    painter->setPen(QColor(55,55,55));
    QLinearGradient gradient(boundingRect().topRight(),boundingRect().bottomRight());
    if(current)
    {
        gradient.setColorAt(0.0f,QColor(255,255,255,125));
        gradient.setColorAt(0.5f,QColor(255,255,255,0));
        gradient.setColorAt(1.0f,QColor(255,255,255,125));
    }
    else if(hover)
    {
        gradient.setColorAt(0.0f,QColor(255,255,255,255));
        gradient.setColorAt(0.3f,QColor(255,255,255,0));
        gradient.setColorAt(0.7f,QColor(255,255,255,0));
        gradient.setColorAt(1.0f,QColor(255,255,255,255));
    }
    else
    {
        gradient.setColorAt(0.0f,QColor(255,255,255,0));
        gradient.setColorAt(0.2f,QColor(255,255,255,55));
        gradient.setColorAt(1.0f,QColor(255,255,255,155));
    }
    painter->drawRoundedRect(boundingRect(),12.0f,12.0f);
    painter->setBrush(gradient);
    painter->drawRoundedRect(boundingRect(),12.0f,12.0f);

    if(current || hover)
        painter->setPen(Qt::darkGray);
    else
        painter->setPen(QColor(220,220,220));


    QFontMetrics metrics(painter->font());
    QString drawnText = metrics.elidedText(myTrackInfo.title,Qt::ElideMiddle,
                                           static_cast<int>(boundingRect().width()) -
                                           metrics.width(myTrackInfo.runTime) - 24);
    painter->drawText(8,0,static_cast<int>(boundingRect().width()) -
                      metrics.width(myTrackInfo.runTime) - 8,boundingRect().height(),
                      Qt::AlignLeft|Qt::AlignVCenter,drawnText);
    painter->drawText(boundingRect().width() - metrics.width(myTrackInfo.runTime) - 8,0,
                      metrics.width(myTrackInfo.runTime),boundingRect().height(),
                      Qt::AlignRight|Qt::AlignVCenter,myTrackInfo.runTime);
}

void PlayListItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
    event->accept();
    dragStart = event->pos();
}

void PlayListItem::mouseDoubleClickEvent(QGraphicsSceneMouseEvent* event)
{
    event->accept();
    if(event->button() == Qt::LeftButton)
        emit clicked(this);
    if(event->button() == Qt::RightButton)
        emit remove(this);
}

void PlayListItem::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
{
    event->accept();
    if((event->pos() - dragStart).manhattanLength() < QApplication::startDragDistance() &&
       event->button() != Qt::LeftButton)
        return;
    QDrag* drag = new QDrag(event->widget());
    QMimeData *mime = new QMimeData;
    drag->setMimeData(mime);
    QByteArray array;
    QDataStream stream(&array,QIODevice::WriteOnly);
    stream << myTrackInfo;
    mime->setData("VulcanoPlayer/PlayListItem",array);
    QPixmap dragPixmap(":/eyecandy/track.png");
    drag->setPixmap(dragPixmap);
    drag->setHotSpot(QPoint(dragPixmap.width()/2,
                            dragPixmap.height()/2));
    drag->exec();
}

PlayListItem::PlayListItem(qreal width,TrackInfo info,QGraphicsItem* parent)
    :ColectionItem(width,0,"",parent),myTrackInfo(info),current(false)
{
    setCursor(QCursor(Qt::PointingHandCursor));
}
