#include "SeekerItem.hpp"
#include <QPainter>
#include <QLinearGradient>
#include <QGraphicsSceneMouseEvent>
#include <QCursor>

QRectF SeekerItem::boundingRect() const
{
    return QRectF(0.0f,0.0f,myWidth,20.0f);
}

void SeekerItem::changeWidth(qreal width)
{
    myWidth = width;
    clipPath = QPainterPath();
    clipPath.addRoundedRect(boundingRect(),10.0f,10.0f);
    update();
}

void SeekerItem::paint(QPainter* painter,const QStyleOptionGraphicsItem*,QWidget*)
{
    painter->fillRect(boundingRect(),QColor(10,10,10));
    painter->setRenderHint(QPainter::Antialiasing);

    QLinearGradient gradient(boundingRect().topRight(),boundingRect().bottomRight());
    gradient.setColorAt(0.0f,QColor(255,255,255,125));
    gradient.setColorAt(0.45f,QColor(255,255,255,75));
    gradient.setColorAt(0.75f,QColor(0,0,0,0));
    painter->setBrush(gradient);
    painter->drawPath(clipPath);
    painter->setClipPath(clipPath);
    QLinearGradient gradient2(boundingRect().topLeft(),boundingRect().bottomLeft());
    gradient2.setColorAt(0.0f,QColor(255,255,255,225));
    gradient2.setColorAt(0.5f,QColor(255,255,255,0));
    gradient2.setColorAt(1.0f,QColor(255,255,255,100));
    painter->setBrush(gradient2);
    painter->setPen(Qt::transparent);

    if(trackLeanght == 0)
        return;
    painter->drawRoundedRect(0,0,boundingRect().width()
                             * (static_cast<float>(currentTime)/static_cast<float>(trackLeanght)),
                             boundingRect().height(),10.0f,10.0f);
    painter->setClipping(false);
    painter->setBrush(Qt::transparent);
    QPen pen;
    pen.setWidth(2.0f);
    pen.setColor(QColor(10,10,10));
    painter->setPen(pen);
    painter->drawPath(clipPath);
}

void SeekerItem::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
    event->accept();
    if(trackLeanght == 0)
        return;
    qint64 mSecPP = trackLeanght/static_cast<qint64>(myWidth);
    qint64 jumpTo = static_cast<qint64>(event->scenePos().x()) * mSecPP;
    emit tryToJup(jumpTo);
    currentTimeChanged(jumpTo);
}

void SeekerItem::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
{
    event->accept();
    if(trackLeanght == 0)
        return;
    qint64 mSecPP = trackLeanght/static_cast<qint64>(myWidth);
    qint64 jumpTo = static_cast<qint64>(event->scenePos().x()) * mSecPP;
    emit tryToJup(jumpTo);
    currentTimeChanged(jumpTo);
}

void SeekerItem::trackLeanghtChanged(qint64 leangh)
{
    trackLeanght = leangh;
    if(trackLeanght == 0)
        trackLeanght = 1;
    currentTime = 0;
    update();
}

void SeekerItem::currentTimeChanged(qint64 current)
{
    currentTime = current;
    if(currentTime > trackLeanght)
        currentTime = trackLeanght;
    update();
}

SeekerItem::SeekerItem(qreal width,QGraphicsItem* parent)
    :QGraphicsObject(parent),myWidth(width)
{
    currentTime = 0;
    trackLeanght = 1;
    clipPath.addRoundedRect(boundingRect(),10.0f,10.0f);
    setCursor(QCursor(Qt::PointingHandCursor));
}
