#ifndef VULANOWINDOW_HPP
#define VULANOWINDOW_HPP

#include "ui_VulcanoWindow.h"
#include <phonon/phononnamespace.h>
#include <phonon/mediasource.h>
#include <phonon/mediaobject.h>
#include <phonon/audiooutput.h>
#include "Colection/TrackInfo.hpp"

class ColectionToolBar;
class GraphicsToolButton;
class Colection;
class ColectionItemDisplay;
class FilterInput;
class PlayListDisplay;
class ArtistAlbumItem;
class SeekerItem;

class VulcanoWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit VulcanoWindow(QWidget *parent = 0);
    ~VulcanoWindow();

protected:
    void changeEvent(QEvent *e);
    void closeEvent(QCloseEvent*);
    void resizeEvent(QResizeEvent*);
    void showEvent(QShowEvent *);

private slots:
    void scan();
    void colectionReady();
    void gotAlbumsArtis(QList<QPair<int,QString> > list);
    void gotTracks(QList<QPair<int,QString> > list);
    void gotTrackInfo(TrackInfo info);
    void gotTracksInfo(TrackInfoList list);
    void gotTrackRequest(int index,int dropIndex);
    void gotTracksRequest(bool album,int index,int dropIndex);

    void artistAlbumItemClocked(int index);
    void trackItemClicked(int index);
    void switchSorting();
    void playFile(QString path);

    void play();
    void stop();
    void pause();

    void next();
    void previous();

    void aboutToFinish();

    void mediaStateChanged(Phonon::State state,Phonon::State oldState);

signals:
    void add(TrackInfoList list);
    void getArtist();
    void getAblums();
    void getTracksFromArtist(int indxe);
    void getTracksFromAlbum(int index);
    void getTrackInfo(int index);
    void getTracksInfoFromArtist(int index);
    void getTracksInfoFromAlbum(int index);

private:
    void createToolButtons();
    void makeScenes();

    Ui::VulanoWindow ui;

    Phonon::MediaObject* mediaObject;
    Phonon::AudioOutput* audioOutPut;

    ColectionToolBar* colectionBar;

    GraphicsToolButton* searchButton;
    Colection* colection;
    FilterInput* artisAlbumFilter;
    FilterInput* trackFilter;
    ArtistAlbumItem* artistAlbumColectionItem;
    ColectionItemDisplay* artistAlbumDisplay;
    ColectionItemDisplay* trackDisplay;
    SeekerItem* seeker;
    PlayListDisplay* playList;

    bool album;
    int dropIndex;
};

#endif // VULANOWINDOW_HPP
